--[[
This class provides an AceModuleCore framework for defining UnitStatus
extensions.  A UnitState module can be defined based on this framework
and be used by UAB (and a UAB user) to recognize when specific unit status
events have occurred.
--]]
uab.modules.UnitStatus = uab:NewModule("UnitStatus", "AceModuleCore-2.0")
uab.modules.UnitStatus:SetModuleMixins("AceDebug-2.0", "AceModuleCore-2.0")
uab.modules.UnitStatus.core = uab
uab.modules.UnitStatus.unitstatus = {}

uabUnitStatus = uab.modules.UnitStatus

function uabUnitStatus:GetAllUnitStatus()
	return self.unitstatus
end

function uabUnitStatus:RegisterUnitStatus(menuname)
	self.unitstatus[menuname] = true
end

function uabUnitStatus:StatusPriority(name)
	local module = self:GetUnitStatus(name)
	
	return module and module:GetPriority(name)
end

function uabUnitStatus:StatusColor(name)
	local module = self:GetUnitStatus(name)
	
	return module and module:GetColor(name)
end

function uabUnitStatus:IsStatusGained(name, unit, action)
	local module = self:GetUnitStatus(name)
	
	return module and module:IsStatusGained(name, unit, action)
end

function uabUnitStatus:GetUnitStatus(name)
	for _, module in self:IterateModules() do
        if module:IsUnitStatus(name) then
			if module:IsEnabled(name) then
                return module
            end
		end
	end
end

uabUnitStatus.modulePrototype.core = uabUnitStatus
uabUnitStatus.modulePrototype.statusgained = {}
uabUnitStatus.modulePrototype.unitStatusDefaultDB = {
    disabled = nil,
    priority = 50,
    color = { r = 1, g = 1, b = 1, a = 1, }
}

function uabUnitStatus.modulePrototype:OnInitialize()
	if not self.db then
		self.core.core:RegisterDefaults(self.name, "profile", self.defaultDB or self.unitStatusDefaultDB)
		self.db = self.core.core:AcquireDBNamespace(self.name)
	end

    self.debugging = self.db.profile.debug

    self.core.core.options.args.unitstatus.args[self.name] = self:CreateUnitStatusOptions()
end

function uabUnitStatus.modulePrototype:RegisterUnitStatus(menuname)
	uabUnitStatus:RegisterUnitStatus(menuname)
end

function uabUnitStatus.modulePrototype:IsEnabled(name)
    if not self.db.profile.disabled then
        local options = self:OptionsForUnitStatus(name)
        
        if options and not options.disabled then
            return true
        end
	end    
end

function uabUnitStatus.modulePrototype:IsUnitStatus(name)
    if not self.db.profile.disabled then
        local options = self:OptionsForUnitStatus(name)
        
        if options and not options.disabled then
            return true
        end
    end
end

function uabUnitStatus.modulePrototype:GetPriority(name)
    if not self.db.profile.disabled then
        local options = self:OptionsForUnitStatus(name)

        if options and not options.disabled then
            return options.priority
		end
	end
end

function uabUnitStatus.modulePrototype:GetColor(name)
    if not self.db.profile.disabled then
        local options = self:OptionsForUnitStatus(name)

        if options and not options.disabled then
            return options.color
		end
	end
end

function uabUnitStatus.modulePrototype:OptionsForUnitStatus(name)
	if name == self.name then
		return self.db.profile
	end
end

function uabUnitStatus.modulePrototype:IsStatusGained(name, unit, action)
	if name == self.name then
		return self.statusgained[unit]
	end
end

function uabUnitStatus.modulePrototype:UnitStatusGained(name, unit)
	if name == self.name then
		self.statusgained[unit] = true
	end
end

function uabUnitStatus.modulePrototype:UnitStatusLost(name, unit)
	if name == self.name then
		self.statusgained[unit] = nil
	end
end

function uabUnitStatus.modulePrototype:CreateUnitStatusOptions()
    return self:CreateUnitStatusOption(self.db.profile, self.name, self.name)
end

function uabUnitStatus.modulePrototype:CreateUnitStatusOption(profile, name, displayname)
    local options
    local me = self

    if not name then
        name = self.name
    end

    if not displayname then
        displayname = name
    end

    options = {
        type = 'group',
        name = name,
        desc = "Unit status settings for the "..displayname.." unit status",
        args = {
            enabled = {
                type = 'toggle',
                name = "Enabled",
                desc = "When checked, enable the use of the "..displayname.." unit status",
                order = 1,
                get = function()
                    return not profile.disabled
                end,
                set = function(v)
                    profile.disabled = not v
                end,
            },
            
            spacing3 = {
                name = " ",
                type = 'header',
                order = 5,
            },

            priority = {
                type = 'range',
                name = "Priority",
                desc = displayname .. " priority",
                disabled = function() return name == "default" end,
                order = 10,
                min = 1,
                max = 100,
                step = 1,
                isPercent = false,
                get = function()
                    return profile.priority
                end,
                set = function(value)
                    profile.priority = value
                end,
            },

            highlight = {
                type = 'color',
                name = "Border highlight color",
                desc = "Set the border hightlight color when "..displayname.." is in effect",
                order = 20,
                hasAlpha = true,
                get = function()
                    local c = profile.color
                    return c.r, c.g, c.b, c.a
                end,
                set = function(r, g, b, a)
                    local c = profile.color
                    c.r, c.g, c.b, c.a = r, g, b, a
                end,
            },
        },
    }
    
    return options
end
